"use strict";

exports.__esModule = true;
exports.dynamicSort = dynamicSort;
exports.selectFilterProps = selectFilterProps;
exports.selectQueryProps = selectQueryProps;
exports.getFilteredList = getFilteredList;
exports.getList = getList;
exports.SELECT_FIELD_TYPE = exports.ARRAY_TYPE = exports.TIME_TYPE = exports.DATE_TYPE = exports.NUMBER_TYPE = exports.STRING_TYPE = void 0;

function _createForOfIteratorHelperLoose(o) { var i = 0; if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) { if (Array.isArray(o) || (o = _unsupportedIterableToArray(o))) return function () { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }; throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } i = o[Symbol.iterator](); return i.next.bind(i); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

var STRING_TYPE = 'string';
exports.STRING_TYPE = STRING_TYPE;
var NUMBER_TYPE = 'number';
exports.NUMBER_TYPE = NUMBER_TYPE;
var DATE_TYPE = 'date';
exports.DATE_TYPE = DATE_TYPE;
var TIME_TYPE = 'time';
exports.TIME_TYPE = TIME_TYPE;
var ARRAY_TYPE = 'array';
exports.ARRAY_TYPE = ARRAY_TYPE;
var SELECT_FIELD_TYPE = 'select_field';
exports.SELECT_FIELD_TYPE = SELECT_FIELD_TYPE;

function getValue(source, fieldName, getSourceValue, isCaseSensitive, type) {
  if (getSourceValue === void 0) {
    getSourceValue = function getSourceValue(fieldValue) {
      return fieldValue;
    };
  }

  if (source != null && getSourceValue(source)) {
    var fieldValue = getSourceValue(source)[fieldName];

    if (type === 'object') {
      return fieldValue ? fieldValue : {};
    }

    if (type === 'number') {
      return fieldValue ? parseFloat(fieldValue) : 0;
    }

    if (typeof fieldValue === 'object' || fieldValue instanceof Object) {
      if (fieldValue.hasOwnProperty('label')) {
        fieldValue = fieldValue.label;
      }
    }

    if (type === 'date') {
      return new Date(fieldValue).setHours(0, 0, 0, 0);
    } else if (type === 'bool') {
      return fieldValue === undefined ? 'false' : fieldValue;
    } else {
      return isCaseSensitive === true ? String(fieldValue) : String(fieldValue).toUpperCase();
    }
  }
}

function dynamicSort(sortField, sortOrientation, getSourceValue) {
  var sortOrder = sortOrientation ? 1 : -1;
  return function (x, y) {
    var a = getValue(x, sortField, getSourceValue);
    var b = getValue(y, sortField, getSourceValue);
    var result = a < b ? -1 : a > b ? 1 : 0;
    return result * sortOrder;
  };
}

function selectFilterProps(filterName, filters) {
  var isOpen = false;
  var hasFilters = false;
  var sortField = null;
  var sortOrientation = true;
  var queries = [];
  var searchValue = null;

  if (filters !== undefined && filters[filterName] !== undefined) {
    var filter = filters[filterName];
    isOpen = filter.isOpen !== undefined ? filter.isOpen : isOpen;
    hasFilters = filter.queries !== undefined ? filter.queries.length : hasFilters;
    sortField = filter.sortField !== undefined ? filter.sortField : sortField;
    sortOrientation = filter.sortOrientation !== undefined ? filter.sortOrientation : sortOrientation;
    queries = filter.queries !== undefined ? filter.queries : queries;
    searchValue = filter.search !== undefined ? filter.search.value : searchValue;
  }

  return {
    isOpen,
    hasFilters,
    sortField,
    sortOrientation,
    queries,
    searchValue
  };
}

function selectQueryProps(query) {
  var value = '';
  var operator;
  var field;
  var type = 'string';
  var isCaseSensitive = false;
  var isSet = false;

  if (query !== undefined) {
    value = query.value !== undefined ? query.value : value;
    operator = query.operator !== undefined ? query.operator : operator;
    field = query.field !== undefined ? query.field : field;
    type = query.type !== undefined ? query.type : type;
    isCaseSensitive = query.isCaseSensitive !== undefined ? query.isCaseSensitive : isCaseSensitive;
    isSet = field !== undefined && field !== null && operator !== undefined && operator !== null && value !== undefined;
  }

  return {
    value,
    operator,
    field,
    type,
    isCaseSensitive,
    isSet
  };
}

function getFilteredList(filterName, filters, list, getSourceValue) {
  var _selectFilterProps = selectFilterProps(filterName, filters),
      sortField = _selectFilterProps.sortField,
      sortOrientation = _selectFilterProps.sortOrientation,
      queries = _selectFilterProps.queries,
      searchValue = _selectFilterProps.searchValue;

  var dateOptions = {
    day: '2-digit',
    month: '2-digit',
    year: 'numeric'
  };

  if (list == null || list.length < 1) {
    return [];
  }

  var result = [].concat(list);
  result = result.filter(function (row, i) {
    var show = true;

    if (queries) {
      for (var _iterator = _createForOfIteratorHelperLoose(queries), _step; !(_step = _iterator()).done;) {
        var query = _step.value;

        var _selectQueryProps = selectQueryProps(query),
            value = _selectQueryProps.value,
            operator = _selectQueryProps.operator,
            field = _selectQueryProps.field,
            isCaseSensitive = _selectQueryProps.isCaseSensitive,
            isSet = _selectQueryProps.isSet,
            type = _selectQueryProps.type;

        if (isSet) {
          var fieldValue = getValue(row, field.value, getSourceValue, isCaseSensitive, type);

          if (type === 'date') {
            var queryDate = moment(value);

            switch (operator.value) {
              case '=':
                show = queryDate.isSame(fieldValue, 'day');
                break;

              case '!=':
                show = !queryDate.isSame(fieldValue, 'day');
                break;

              case '>':
                show = queryDate.isBefore(fieldValue, 'day');
                break;

              case '>=':
                show = queryDate.isSameOrBefore(fieldValue, 'day');
                break;

              case '<':
                show = queryDate.isAfter(fieldValue, 'day');
                break;

              case '<=':
                show = queryDate.isSameOrAfter(fieldValue, 'day');
                break;

              default:
                break;
            }

            if (!show) {
              return show;
            }
          } else if (type === 'number') {
            var queryNumber = parseFloat(value);

            switch (operator.value) {
              case '=':
                show = fieldValue === queryNumber;
                break;

              case '!=':
                show = fieldValue !== queryNumber;
                break;

              case '>':
                show = fieldValue > queryNumber;
                break;

              case '>=':
                show = fieldValue >= queryNumber;
                break;

              case '<':
                show = fieldValue < queryNumber;
                break;

              case '<=':
                show = fieldValue <= queryNumber;
                break;

              default:
                break;
            }

            if (!show) {
              return show;
            }
          } else if (type === 'bool') {
            var fieldVal = false;

            if (fieldValue === true || fieldValue === 'true') {
              fieldVal = true;
            }

            var queryVal = false;

            if (value === true || value === 'true') {
              queryVal = true;
            }

            show = fieldVal === queryVal;

            if (!show) {
              return show;
            }
          } else if (type === 'object') {
            show = JSON.stringify(fieldValue ? fieldValue : '').toUpperCase().indexOf(String(value ? value : '').toUpperCase()) !== -1;

            if (!show) {
              return show;
            }
          } else {
            var valueString = String(value);
            var fieldValueString = String(fieldValue);
            var queryValueString = isCaseSensitive === true ? valueString : valueString.toUpperCase();

            switch (operator.value) {
              case 'like':
                show = fieldValueString.indexOf(queryValueString) !== -1;
                break;

              case 'notlike':
                show = fieldValueString.indexOf(queryValueString) === -1;
                break;

              case '=':
                show = fieldValueString === queryValueString;
                break;

              case '>':
                show = fieldValueString.localeCompare(queryValueString) > 0;
                break;

              case '>=':
                show = fieldValueString.localeCompare(queryValueString) >= 0;
                break;

              case '<':
                show = fieldValueString.localeCompare(queryValueString) < 0;
                break;

              case '<=':
                show = fieldValueString.localeCompare(valueString) <= 0;
                break;

              default:
                break;
            }

            if (!show) {
              return show;
            }
          }
        }
      }
    }

    if (searchValue != null && searchValue !== '' && show) {
      show = JSON.stringify(row).toUpperCase().indexOf(String(searchValue).toUpperCase()) !== -1;
    }

    return show;
  });

  if (result !== undefined && sortField !== null) {
    result.sort(dynamicSort(sortField.value, sortOrientation, getSourceValue));
  }

  return result;
}

function getList(filter, list) {
  if (filter === void 0) {
    filter = {};
  }

  var result = [].concat(list);
  var _filter = filter,
      _filter$queries = _filter.queries,
      queries = _filter$queries === void 0 ? [] : _filter$queries,
      _filter$fields = _filter.fields,
      fields = _filter$fields === void 0 ? [] : _filter$fields;
  console.log('queries', queries);

  if (list == null || list.length < 1) {
    return [];
  }

  return result;
}